/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Serializer.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_Serializer_H
#define OpenSOAP_Serializer_H

#include <OpenSOAP/String.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

    /* OpenSOAP Serializer Function Type */
    typedef int
/*    OPENSOAP_API */
    (*OpenSOAPSerializerFunc)(void * /* [in] */from_value,
                              OpenSOAPStringPtr /* [out] */to);

    /* OpenSOAP Deserializer Function Type */
    typedef int
/*    OPENSOAP_API */
    (*OpenSOAPDeserializerFunc)(OpenSOAPStringPtr /* [in] */from,
                                void * /* [out] */to_value);


    /* Register Serializer */
    int
    OPENSOAP_API
    OpenSOAPSerializerRegisterMB(const char * /* [in] */ soap_typename,
                                 OpenSOAPSerializerFunc /* [in] */ serializer,
                                 OpenSOAPDeserializerFunc /* [in] */ deserializer);

    /* Register Serializer */
    int
    OPENSOAP_API
    OpenSOAPSerializerRegisterWC(const wchar_t * /* [in] */ soap_typename,
                                 OpenSOAPSerializerFunc /* [in] */ serializer,
                                 OpenSOAPDeserializerFunc /* [in] */ deserializer);

    /* Get Serializer(OpenSOAPString) */
    int
    OPENSOAP_API
    OpenSOAPGetSerializer(OpenSOAPStringPtr /* [in] */ soap_typename,
                          OpenSOAPSerializerFunc * /* [out] */ serializer);

    /* Get Serializer(MB) */
    int
    OPENSOAP_API
    OpenSOAPGetSerializerMB(const char * /* [in] */ soap_typename,
                            OpenSOAPSerializerFunc * /* [out] */ serializer);

    /* Get Serializer(WC) */
    int
    OPENSOAP_API
    OpenSOAPGetSerializerWC(const wchar_t * /* [in] */ soap_typename,
                            OpenSOAPSerializerFunc * /* [out] */ serializer);
    
    /* Get Deserializer(OpenSOAPString) */
    int
    OPENSOAP_API
    OpenSOAPGetDeserializer(OpenSOAPStringPtr /* [in] */ soap_typename,
                            OpenSOAPDeserializerFunc * /* [out] */ deserializer);
    /* Get Deserializer(MB) */
    int
    OPENSOAP_API
    OpenSOAPGetDeserializerMB(const char * /* [in] */ soap_typename,
                              OpenSOAPDeserializerFunc * /* [out] */ deserializer);

    /* Get Deserializer(WC) */
    int
    OPENSOAP_API
    OpenSOAPGetDeserializerWC(const wchar_t * /* [in] */ soap_typename,
                              OpenSOAPDeserializerFunc * /* [out] */ deserializer);

    /* default serializer regist */
    int
    OPENSOAP_API
    OpenSOAPSerializerRegistDefaults(void);
    
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_Serializer_H */
