/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: String.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_String_H
#define OpenSOAP_String_H

#include <OpenSOAP/ByteArray.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

    typedef struct tagOpenSOAPString OpenSOAPString;
    typedef OpenSOAPString    *OpenSOAPStringPtr;


    int
    OPENSOAP_API
    OpenSOAPStringCreate(OpenSOAPStringPtr * /* [out] */ str);
    
    /* MultiByte str initialize */
    int
    OPENSOAP_API
    OpenSOAPStringCreateWithMB(const char * /* [in] */ mb_str,
                               OpenSOAPStringPtr * /* [out] */ str);

    /* wchar_t str initialize */
    int
    OPENSOAP_API
    OpenSOAPStringCreateWithWC(const wchar_t * /* [in] */ wc_str,
                               OpenSOAPStringPtr * /* [out] */ str);

    /* char encoding str initialize */
    int
    OPENSOAP_API
    OpenSOAPStringCreateWithCharEncodingString(const char *
                                               /* [in] */ char_enc,
                                               OpenSOAPByteArrayPtr
                                               /* [in] */ char_enc_str,
                                               OpenSOAPStringPtr *
                                               /* [out] */ str);
    
    /* Use OpenSOAPString Resource retain.
     * this term from Objective-C */
    int
    OPENSOAP_API
    OpenSOAPStringRetain(OpenSOAPStringPtr /* [in] */ str);

    /* release OpenSOAPString Resource */
    int
    OPENSOAP_API
    OpenSOAPStringRelease(OpenSOAPStringPtr /* [in] */ str);

    /* get length for MultiByte */
    int
    OPENSOAP_API
    OpenSOAPStringGetLengthMB(OpenSOAPStringPtr /* [in] */ str,
                              size_t * /* [out] */ len);
        
    /* get length for wchar_t */
    int
    OPENSOAP_API
    OpenSOAPStringGetLengthWC(OpenSOAPStringPtr /* [in] */ str,
                              size_t * /* [out] */ len);

	/* get MultiByte string using allocator */
	int
	OPENSOAP_API
	OpenSOAPStringGetStringMBWithAllocator(/* [in]  */  OpenSOAPStringPtr
										   str,
										   /* [in]  */  void *
										   (*allocator)(size_t),
										   /* [out] */ size_t *len,
										   /* [out] */ char **mbStr);

	/* get wchar_t string using allocator */
	int
	OPENSOAP_API
	OpenSOAPStringGetStringWCWithAllocator(/* [in]  */  OpenSOAPStringPtr
										   str,
										   /* [in]  */  void *
										   (*allocator)(size_t),
										   /* [out] */ size_t *len,
										   /* [out] */ wchar_t **mbStr);

    /* get MultiByte string */
    int
    OPENSOAP_API
    OpenSOAPStringGetStringMB(OpenSOAPStringPtr /* [in] */ str,
                              size_t * /* [in, out] */ len,
                              char * /* [out] */ mb_str);
        
    /* get wchar_t string */
    int
    OPENSOAP_API
    OpenSOAPStringGetStringWC(OpenSOAPStringPtr /* [in] */ str,
                              size_t * /* [in, out] */ len,
                              wchar_t * /* [out] */ wc_str);

    /* get char encoding string */
    int
    OPENSOAP_API
    OpenSOAPStringGetCharEncodingString(OpenSOAPStringPtr /* [in] */ str,
                                        const char * /* [in] */ char_enc,
                                        OpenSOAPByteArrayPtr /* [out] */
                                        char_enc_str);
    
    /* set MultiByte string */
    int
    OPENSOAP_API
    OpenSOAPStringSetStringMB(OpenSOAPStringPtr /* [in] */ str,
                              const char * /* [in] */ mb_str);
        
    /* set wchar_t string */
    int
    OPENSOAP_API
    OpenSOAPStringSetStringWC(OpenSOAPStringPtr /* [in] */ str,
                              const wchar_t * /* [in] */ wc_str);

    /* set char encoding string */
    int
    OPENSOAP_API
    OpenSOAPStringSetCharEncodingString(OpenSOAPStringPtr /* [in, out] */ str,
                                        const char * /* [in] */ char_enc,
                                        OpenSOAPByteArrayPtr /* [in] */
                                        char_enc_str);
    /* format string */
    int
    OPENSOAP_API
    OpenSOAPStringFormatMB(OpenSOAPStringPtr /* [in] */ str,
                           const char * /* [in] */ format, ...);

    /* format string */
    int
    OPENSOAP_API
    OpenSOAPStringFormatWC(OpenSOAPStringPtr /* [in] */ str,
                           const wchar_t * /* [in] */ format, ...);

    /* compare multibyte character string */
    int
    OPENSOAP_API
    OpenSOAPStringCompareMB(OpenSOAPStringPtr /* [in] */ str,
                            const char * /* [in] */ cmp_str,
                            int * /* [out] */ cmp_rslt);

    /* compare wide character string */
    int
    OPENSOAP_API
    OpenSOAPStringCompareWC(OpenSOAPStringPtr /* [in] */ str,
                            const wchar_t * /* [in] */ cmp_str,
                            int * /* [out] */ cmp_rslt);

    /* compare string */
    int
    OPENSOAP_API
    OpenSOAPStringCompare(OpenSOAPStringPtr /* [in] */ str,
                          OpenSOAPStringPtr /* [in] */ cmp_str,
                          int * /* [out] */ cmp_rslt);

    /* find string */
    int
    OPENSOAP_API
    OpenSOAPStringFindStringMB(OpenSOAPStringPtr /* [in] */ str,
                               const char * /* [in] */ find_str,
                               size_t * /* [in, out] */ idx);

    /* find string */
    int
    OPENSOAP_API
    OpenSOAPStringFindStringWC(OpenSOAPStringPtr /* [in] */ str,
                               const wchar_t * /* [in] */ find_str,
                               size_t * /* [in, out] */ idx);

    /* find string */
    int
    OPENSOAP_API
    OpenSOAPStringFindString(OpenSOAPStringPtr /* [in] */ str,
                             OpenSOAPStringPtr /* [in] */ find_str,
                             size_t * /* [in, out] */ idx);

    /* find string condition function */
    typedef int
    (*OpenSOAPStringFindIfFunc)(void * /* [in, out] */ opt,
                                wchar_t /* [in] */ c,
                                int * /* [out] */ judge);
    /* find string with condition function */
    extern
    int
    OPENSOAP_API
    OpenSOAPStringFindIfStringIndex(OpenSOAPStringPtr /* [in] */ str,
                                    OpenSOAPStringFindIfFunc /* [in] */
                                    find_func,
                                    void * /* [in] */ find_func_opt,
                                    size_t * /* [in, out] */ idx);
    /* replace string */
    int
    OPENSOAP_API
    OpenSOAPStringReplaceStringMB(OpenSOAPStringPtr /* [in, out] */ str,
                                  const char * /* [in] */ find_str,
                                  const char * /* [in] */ rplc_str,
                                  size_t * /* [in, out] */ idx);
    
    /* replace string */
    int
    OPENSOAP_API
    OpenSOAPStringReplaceStringWC(OpenSOAPStringPtr /* [in, out] */ str,
                                  const wchar_t * /* [in] */ find_str,
                                  const wchar_t * /* [in] */ rplc_str,
                                  size_t * /* [in, out] */ idx);
    
    /* replace string */
    int
    OPENSOAP_API
    OpenSOAPStringReplaceString(OpenSOAPStringPtr /* [in, out] */ str,
                                OpenSOAPStringPtr /* [in] */ find_str,
                                OpenSOAPStringPtr /* [in] */ rplc_str,
                                size_t * /* [in, out] */ idx);
    
    /* clear OpenSOAPString */
    int
    OPENSOAP_API
    OpenSOAPStringClear(OpenSOAPStringPtr str);

    /* concat MB string to OpenSOAPString */
    int
    OPENSOAP_API
    OpenSOAPStringAppendMB(OpenSOAPStringPtr str,
                           const char *mb_str,
                           size_t       mb_len);

    /* concat MB string to OpenSOAPString */
    int
    OPENSOAP_API
    OpenSOAPStringAppendWC(OpenSOAPStringPtr str,
                           const wchar_t *wc_str,
                           size_t   wc_len);

    /* duplicate string */
    int
    OPENSOAP_API
    OpenSOAPStringDuplicate(OpenSOAPStringPtr /* [in] */ str,
                            OpenSOAPStringPtr * /* [out] */ dup_str);

    /* get substring */
    int
    OPENSOAP_API
    OpenSOAPStringGetSubstring(OpenSOAPStringPtr /* [in] */ str,
                               size_t   /* [in] */ beg,
                               size_t   /* [in] */ len,
                               OpenSOAPStringPtr * /* [out] */ sub_str);
    /*
     * this function isn't include OpenSOAPString member,
     * but this has something to do with String processing.
     */
    int
    OPENSOAP_API
    OpenSOAPStringConvertCharEncoding(const char * /* [in] */ from_enc,
                                      OpenSOAPByteArrayPtr /* [in] */ from_str,
                                      const char * /* [in] */ to_enc,
                                      OpenSOAPByteArrayPtr /* [out] */ to_str);
    
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_String_H */
