/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: XMLElm.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_XMLElm_H
#define OpenSOAP_XMLElm_H

#include <OpenSOAP/XMLAttr.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

    typedef struct tagOpenSOAPXMLElm OpenSOAPXMLElm;
    typedef OpenSOAPXMLElm *OpenSOAPXMLElmPtr;

    int
    OPENSOAP_API
    OpenSOAPXMLElmCreate(OpenSOAPXMLElmPtr * /* [out] */ elm);

    int
    OPENSOAP_API
    OpenSOAPXMLElmSetNamespaceMB(OpenSOAPXMLElmPtr /* [in, out] */ elm,
                                 const char * /* [in] */ ns_uri,
                                 const char * /* [in] */ ns_prefix);

    int
    OPENSOAP_API
    OpenSOAPXMLElmSetNamespaceWC(OpenSOAPXMLElmPtr /* [in, out] */ elm,
                                 const wchar_t * /* [in] */ ns_uri,
                                 const wchar_t * /* [in] */ ns_prefix);

    int
    OPENSOAP_API
    OpenSOAPXMLElmSearchNamespaceMB(OpenSOAPXMLElmPtr /* [in] */ elm,
                                    const char * /* [in] */ ns_uri,
                                    const char * /* [in] */ ns_prefix,
                                    OpenSOAPXMLNamespacePtr * /* [out] */ ns);
    
    int
    OPENSOAP_API
    OpenSOAPXMLElmSearchNamespaceWC(OpenSOAPXMLElmPtr /* [in] */ elm,
                                    const wchar_t * /* [in] */ ns_uri,
                                    const wchar_t * /* [in] */ ns_prefix,
                                    OpenSOAPXMLNamespacePtr * /* [out] */ ns);
    
    int
    OPENSOAP_API
    OpenSOAPXMLElmGetNamespace(OpenSOAPXMLElmPtr /* [in, out] */ elm,
                               OpenSOAPXMLNamespacePtr * /* [out] */ ns);

    int
    OPENSOAP_API
    OpenSOAPXMLElmDefineNamespaceMB(OpenSOAPXMLElmPtr /* [in, out] */ elm,
                                    const char * /* [in] */ ns_uri,
                                    const char * /* [in] */ ns_prefix,
                                    OpenSOAPXMLNamespacePtr * /* [out] */ ns);

    int
    OPENSOAP_API
    OpenSOAPXMLElmDefineNamespaceWC(OpenSOAPXMLElmPtr /* [in, out] */ elm,
                                    const wchar_t * /* [in] */ ns_uri,
                                    const wchar_t * /* [in] */ ns_prefix,
                                    OpenSOAPXMLNamespacePtr * /* [out] */ ns);

    int
    OPENSOAP_API
    OpenSOAPXMLElmAddAttributeMB(OpenSOAPXMLElmPtr /* [in, out] */ elm,
                                 const char * /* [in] */ attr_name,
                                 const char * /* [in] */ attr_type,
                                 void * /* [in] */ attr_value,
                                 OpenSOAPXMLAttrPtr * /* [out] */ attr);

    int
    OPENSOAP_API
    OpenSOAPXMLElmAddAttributeWC(OpenSOAPXMLElmPtr /* [in, out] */ elm,
                                 const wchar_t * /* [in] */ attr_name,
                                 const wchar_t * /* [in] */ attr_type,
                                 void * /* [in] */ attr_value,
                                 OpenSOAPXMLAttrPtr * /* [out] */ attr);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetAttributeMB(OpenSOAPXMLElmPtr /* [in, out] */ elm,
                                 const char * /* [in] */ attr_name,
                                 OpenSOAPXMLAttrPtr * /* [out] */ attr);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetAttributeWC(OpenSOAPXMLElmPtr /* [in, out] */ elm,
                                 const wchar_t * /* [in] */ attr_name,
                                 OpenSOAPXMLAttrPtr * /* [out] */ attr);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetNextChild(OpenSOAPXMLElmPtr /* [in] */ elm,
                               OpenSOAPXMLElmPtr * /* [out] */ cld_elm);

    int
    OPENSOAP_API
    OpenSOAPXMLElmAddChildMB(OpenSOAPXMLElmPtr /* [in] */ elm,
                             const char * /* [in] */ cld_name,
                             OpenSOAPXMLElmPtr * /* [out] */ cld_elm);

    int
    OPENSOAP_API
    OpenSOAPXMLElmAddChildWC(OpenSOAPXMLElmPtr /* [in] */ elm,
                             const wchar_t * /* [in] */ cld_name,
                             OpenSOAPXMLElmPtr * /* [out] */ cld_elm);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetChildMB(OpenSOAPXMLElmPtr /* [in] */ elm,
                             const char * /* [in] */ cld_name,
                             OpenSOAPXMLElmPtr * /* [out] */ cld_elm);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetChildWC(OpenSOAPXMLElmPtr /* [in] */ elm,
	                         const wchar_t * /* [in] */ cld_name,
                             OpenSOAPXMLElmPtr * /* [out] */ cld_elm);


    int
    OPENSOAP_API
    OpenSOAPXMLElmGetValueMB(OpenSOAPXMLElmPtr /* [in] */ elm,
                             const char * /* [in] */ type_name,
                             void * /* [out] */ value);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetValueWC(OpenSOAPXMLElmPtr /* [in] */ elm,
                             const wchar_t * /* [in] */ type_name,
                             void * /* [out] */ value);

    int
    OPENSOAP_API
    OpenSOAPXMLElmSetValueMB(OpenSOAPXMLElmPtr /* [in] */ elm,
                             const char * /* [in] */ type_name,
                             void * /* [in] */ value);

    int
    OPENSOAP_API
    OpenSOAPXMLElmSetValueWC(OpenSOAPXMLElmPtr /* [in] */ elm,
                             const wchar_t * /* [in] */ type_name,
                             void * /* [in] */ value);

    int
    OPENSOAP_API
    OpenSOAPXMLElmGetCharEncodingString(OpenSOAPXMLElmPtr /* [in] */ elm,
                                        const char * /* [in] */ ch_enc,
                                        OpenSOAPByteArrayPtr /* [out] */ b_ary);

	/* get XML Element's name as OpenSOAPString */
    int
    OPENSOAP_API
    OpenSOAPXMLElmGetNameString(/* [in]  */ OpenSOAPXMLElmPtr elm,
								/* [out] */ OpenSOAPStringPtr *name);
	

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_XMLElm_H */
