# -----------------------------------------------------------------------------
#   $RCSfile: CalcAsync.mak,v $
#  
#   See Copyright for the status of this software.
#  
#   The OpenSOAP Project
#   http://opensoap.jp/
# -----------------------------------------------------------------------------

PROGRAMS = CalcApp.cgi CalcClient CalcService CalcService.cgi

OPENSOAP_PREFIX=/usr/local
OPENSOAP_INCLUDE=$(OPENSOAP_PREFIX)/include
OPENSOAP_LIB=$(OPENSOAP_PREFIX)/lib

CFLAGS= -I$(OPENSOAP_INCLUDE)
# CFLAGS=-g -I$(OPENSOAP_INCLUDE)
CGI_CFLAGS=$(CFLAGS) -DCONNECT_TYPE_CGI
CLIENT_CGI_CFLAGS=$(CFLAGS) -DCLIENT_CGI
CLIENT_LDFLAGS=-Wl,-rpath -Wl,$(OPENSOAP_LIB) -lOpenSOAPClient
SERVICE_LDFLAGS=$(CLIENT_LDFLAGS) -lOpenSOAPService

SOURCES = CalcApp.c CalcClient.c CalcService.c
OBJS=$(SOURCES:.c=.o) CalcService.cgi.o CalcClient.cgi.o


all: $(PROGRAMS)


## Make Client Programs

CalcClient: CalcClient.o
	$(CC) -o $@ $^ $(CLIENT_LDFLAGS)

CalcApp.cgi: CalcClient.cgi.o CalcApp.o 
	$(CC) -o $@ $^ $(CLIENT_LDFLAGS)

CalcClient.cgi.o: CalcClient.c
	$(CC) $(CLIENT_CGI_CFLAGS) -c -o $@ $^


## Make Service Programs

CalcService: CalcService.o
	$(CC)  -o $@ $^ $(SERVICE_LDFLAGS)

CalcService.cgi: CalcService.cgi.o
	$(CC)  -o $@ $^ $(SERVICE_LDFLAGS)

CalcService.cgi.o: CalcService.c
	$(CC) $(CGI_CFLAGS) -c -o $@ $<


## Install Client and Service Programs

install: $(PROGRAMS)
	cp CalcService.cgi /home/httpd/cgi-bin/
	cp CalcService /usr/local/sbin/
	cp Calc.ssml /var/tmp/OpenSOAP/conf/ssml/
	cp CalcApp.cgi /home/httpd/cgi-bin/
	cp -r ./CalcHtml /home/httpd/cgi-bin/


clean: 
	rm -f $(OBJS) $(PROGRAMS)


## Test Service Programs for Success Cases

stest1:
	cat CalcSampleMessages/CalcAdd.xml | ./CalcService.cgi
stest2:	    
	cat CalcSampleMessages/CalcSubtract.xml | ./CalcService.cgi
stest3:	    
	cat CalcSampleMessages/CalcMultiply.xml | ./CalcService.cgi
stest4:	    
	cat CalcSampleMessages/CalcDivide.xml | ./CalcService.cgi


## Test Client-Service Hookups for Success Cases

ctest1:
	./CalcClient -s http://localhost/cgi-bin/CalcService.cgi -c EUC-JP Add 10 20
ctest2:
	./CalcClient -s http://localhost/cgi-bin/CalcService.cgi -c EUC-JP Subtract10 20
ctest3:
	./CalcClient -s http://localhost/cgi-bin/CalcService.cgi -c EUC-JP Multiply 10 20
ctest4:
	./CalcClient -s http://localhost/cgi-bin/CalcService.cgi -c EUC-JP Divide 10 20


## Test Client-Server-Service Hookups for Sync/Async Success Cases

cstest1:
	./CalcClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP Add 10 20
cstest1a:
	./CalcClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP AddAsync 10 20 > tmp_msg

cstest2:
	./CalcClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP Subtract 10 20
cstest2a:
	./CalcClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP SubtractAsync 10 20 > tmp_msg

cstest3:
	./CalcClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP Multiply 10 20
cstest3a:
	./CalcClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP MultiplyAsync 10 20 > tmp_msg

cstest4:
	./CalcClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP Divide 10 20
cstest4a:
	./CalcClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP DivideAsync 10 20 > tmp_msg

cstestr:
	cat tmp_msg | ./CalcClient -s http://localhost/cgi-bin/soapInterface.cgi -c EUC-JP GetResult
