/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: FIFOManager.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#ifndef _FIFOManager_H_
#define _FIFOManager_H_

#include <string>
#include <list>
#include <pthread.h>

#include "FIFODescriptor.h"

namespace OpenSOAP {

  //class FIFOManager;
  //class FIFOInfo;
  //class FIFOSelector;

  //#####################################################
  // FIFOt@C͏艺̂Qt@CZbg
  // ǗKv邽߁A߂ꂽ
  // "({p|p}vtBbNX){(j[NID)"
  // t@C̖KƂ
  // Ex. ȉ̂Qt@CłPZbg
  //   _UPSIDE_001
  //   _DOWNSIDE_001
  //#####################################################

  //FIFO̊ǗsȂǗNX
  class FIFOManager {

    class FIFOInfo;

  public:
    //FIFOSelectorƂ̒ʐMvgR
    /*
    static const std::string OPEN_UPSIDE_FIFO;
    static const std::string OPEN_DOWNSIDE_FIFO;
    static const std::string CLOSE_UPSIDE_FIFO;
    static const std::string CLOSE_DOWNSIDE_FIFO;
    */
#define OPEN_UPSIDE_FIFO    "OPEN_UPSIDE"
#define OPEN_DOWNSIDE_FIFO  "OPEN_DOWNSIDE"
#define CLOSE_UPSIDE_FIFO   "CLOSE_UPSIDE"
#define CLOSE_DOWNSIDE_FIFO "CLOSE_DOWNSIDE"

  public:
    enum WAIT_OPT { 
      WAIT_NOLIMIT = -1, // max 180sec.
      NO_WAIT      = 0 
    };

  public:
    //FIFOManager(const std::string& fifo_dir_path);
    FIFOManager(const std::string& fifo_dir_path, const int port);
    virtual ~FIFOManager();

  protected:

    FIFOInfo* openUpsideFIFO(const int timeout_second);
    FIFOInfo* openDownsideFIFO(const int timeout_second);

    int closeUpsideFIFO(const std::string& id);
    int closeDownsideFIFO(const std::string& id);

    //int openForUPSIDE(FIFODescriptor& fifo_desc);
    //int openForDOWNSIDE(FIFODescriptor& fifo_desc);


    //wfBNgFIFOt@C擾
    //FIFOInfõXg𐶐()
    void makeFIFOInfo(std::list<string> fifoList);

    pthread_mutex_t fifoInfoColl_mutex_;
    pthread_cond_t  fifoInfoColl_cond_;
    
    std::list<FIFOInfo> fifoInfoColl_;

    static const char FILE_PATH_DELIMIT; // = '/'

    //FIFOSelectorƂ̒ʐM
    typedef struct ThrDataTag {
      FIFOManager* that;
      int sockfd;
    } ThrData;

    static void* connectSelector(void* arg);

    //========== FIFOInfo =============
    //FIFOZbg(艺)Ǘ邽߂̃f[^NX
    class FIFOInfo {

    public:
      static const std::string UPSIDE_FIFO_PREFIX;   // = "_UPSIDE_";
      static const std::string DOWNSIDE_FIFO_PREFIX; // = "_DOWNSIDE_";

    public:
      enum FIFO_STAT { INIT = 0,            //
		       UPSIDE_READY,        //󂯑ڑ
		       DOWNSIDE_CONNECTING, //󂯑ڑ
		       UPSIDE_CLOSED,       //󂯑Ǐ
		       DOWNSIDE_CLOSED,     //󂯑Ǐ
		       BUSY };              //ڑ
    public:
      FIFOInfo(const std::string& id,
		const std::string& fifoUpside,
		const std::string& fifoDownside);
      virtual ~FIFOInfo();

      FIFO_STAT getStatus() const {return fifoStat_;}
      void setStatus(FIFO_STAT stat) {fifoStat_ = stat;}

      FIFODescriptor& getUpsideFIFODescriptor() {return upsideDesc_;}
      FIFODescriptor& getDownsideFIFODescriptor() {return downsideDesc_;}

      const std::string& getUpsideFIFO() const {return fifoUpside_;}
      const std::string& getDownsideFIFO() const {return fifoDownside_;}

      const std::string& getId() const {return upsideDesc_.getId();}

      static bool checkValidFileName(const std::string& fifoFile);

      //fobNp
      void spy() const;

    protected:
      FIFO_STAT fifoStat_;
      FIFODescriptor upsideDesc_;
      FIFODescriptor downsideDesc_;

      std::string fifoUpside_;
      std::string fifoDownside_;

    };
    //========== End of FIFOInfo =============

  };

} // end of namespace OpenSOAP


#endif _FIFOManager_H_
