/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: FIFOSrvDrv.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <string>
#include <vector>

#include <sys/time.h>
#include <unistd.h>

/* server common library */
#include "ServerCommon.h"
#include "ChannelDescriptor.h"
#include "FIFOSrvChnlSelector.h"
#include "ResSpoolPushChnlSelector.h"

/* srvdrv library */
#include "FIFOSrvDrv.h"

/* common library */
#include "connection.h"
#include "FileIDFunc.h"
#include "SOAPMessage.h"
#include "StringUtil.h"
#include "Timeout.h"

/* SSML Information Function library */
#include "SSMLInfoFunc.h"
#include "SSMLInfo.h"



using namespace OpenSOAP;

//----- Constructor -----//
FIFOSrvDrv::FIFOSrvDrv(const ChannelDescriptor& chnlDesc)
  : chnlDesc_(chnlDesc)
{
  cerr << "SrvDrv::SrvDrv" << endl;
  
  pthread_mutex_init(&_mt, NULL);
  downside_timeout = 5;
  _isAsync = false;
}// end of FIFOSrvDrv ()

//----- Destructor -----//
FIFOSrvDrv::~FIFOSrvDrv ()
{
  cerr << "SrvDrv::~SrvDrv" << endl;
  pthread_mutex_destroy(&_mt);
}// end of ~FIFOSrvDrv ()

//----- [J̓Iy[Vs -----//
bool
FIFOSrvDrv::execLocalSyncOperation()
{
  
  struct timeval tv_start, tv_end;
  struct timezone tz;
  
  gettimeofday(&tv_start, &tz);
  cerr << "tv_start.tv_sec  = " << tv_start.tv_sec << endl;
  cerr.flush();
  
  // for debug
  cerr << "FIFOSrvDrv::execLocalSyncOperation() ttl=(" << ttl << ")" << endl;
  
  // for debug
  cout << "FIFOSrvDrv::execLocalSyncOperation() create thread..." << endl;
  
  //ړȈsȂXbh𐶐
  pthread_t thread_t;
  int status = pthread_create(&thread_t, NULL, localSyncThrFunc, this);
  
  // for debug
  cout << "FIFOSrvDrv::execLocalSyncOperation() "
       << "thread create done... status=(" << status << ")" << endl;
  
  //^C}[IuWFNg
  OpenSOAP::Timeout timeout(ttl, thread_t);
  //^CAEgĎJn
  timeout.start();
  
  //ړȈɃXbh
  status = pthread_join(thread_t, NULL);
  
  if (status != 0) {
    cerr << "FIFOSrvDrv::execLocalSyncOperation() pthread join error" << endl;
  }
  
  //ړIIɂ^C}[~
  timeout.finish();
  
  // for debug
  cout << "FIFOSrvDrv::execLocalSyncOperation() bye..." << endl;
  cout << "timeout.targetThreadIsCanceled = " 
       << timeout.targetThreadIsCanceled() << endl;
  
  gettimeofday(&tv_end, &tz);
  cerr << "tv_end.tv_sec  = " << tv_end.tv_sec << endl;
  cerr.flush();
  
  return timeout.targetThreadIsCanceled();
}

//----- [JœIy[Vs߂̃Xbh  -----//
void* 
FIFOSrvDrv::localSyncThrFunc (void* arg) 
{
  FIFOSrvDrv* that = (FIFOSrvDrv*)arg;
  
  // for debug  
  cout << "FIFOSrvDrv::localSyncThrFunc() thr_func wait..." << endl;
  
  // T[rX̕ԓ҂
  
  ChannelDescriptor srvChnlDesc;
  ChannelSelector* fifoSrvChnlSelector = new FIFOSrvChnlSelector();
  if (0 != fifoSrvChnlSelector->open(srvChnlDesc, 10)) {
    cerr << "SrvDrv::run: FIFOSrvChnl open failed" << endl;
    return NULL;
  }
  
  int lock_status = pthread_mutex_lock (&(that->_mt));
  if (0 != lock_status) {
    cerr << "[[Error!!]] void* FIFOSrvDrv::localSyncThrFunc(void*)" << endl;
    cerr << "            pthread_mutex_lock() failed." << endl;
    cerr.flush();
    return NULL;
  }
  
  if (0 > srvChnlDesc.write(that->fileIDOfMsgFromMsgdrv)) {
    cerr << "SrvDrv::run: write failed to FIFOSrv" << endl;
    return NULL;
  }
  cerr << "** SrvDrv ** WRITE TO FIFOSrv=[" 
       << that->fileIDOfMsgFromMsgdrv << "]" << endl;
  
  if (0 > srvChnlDesc.read(that->fileIDOfMsgFromService)) {
    cerr << "SrvDrv::run: read failed from FIFOSrv" << endl;
    return NULL;
  }
  cerr << "** SrvDrv ** READ From FIFOSrv=[" 
       << that->fileIDOfMsgFromService << "]" << endl;
  
  int unlock_status = pthread_mutex_unlock (&(that->_mt));
  if (0 != unlock_status) {
    cerr << "[[Error!!]] void* FIFOSrvDrv::localSyncThrFunc(void*)" << endl;
    cerr << "            pthread_mutex_unlock() failed." << endl;
    cerr.flush();
    return NULL;
  }
  
  fifoSrvChnlSelector->close(srvChnlDesc);
  
  delete fifoSrvChnlSelector;
  
  
  return NULL;
}

void
FIFOSrvDrv::run()
{
  
  cerr << "SVR: read ready." << endl;
  
  if (0 > chnlDesc_.read(fileIDOfMsgFromMsgdrv)) {
    cerr << TAG_ERR << "SrvDrv::run: read failed" << endl;
    return;
  }
  
  cerr << "$$ SrvDrv $$ READ From MsgDrv=[" 
       << fileIDOfMsgFromMsgdrv << "]" << endl;
  
  cerr << "extract Operation and Parameter" << endl;
  std::vector<std::string> operation_names;
  std::vector<std::string> param_names;
  std::vector<std::string> param_values;
  
  soapMsgFromMsgdrv = revertFromFileID(fileIDOfMsgFromMsgdrv);
  
  extractOperationAndParam (soapMsgFromMsgdrv, operation_names, 
                            param_names, param_values);
  
  OpenSOAP::SSMLInfo ssmlInfo;
  ssmlInfo.setOperationName(operation_names[0]);
  bool operationExist = getSSMLInfo(ssmlInfo);
  
  bool soapMsgIsValid = false;
  bool ttlHeaderExist = false;
  std::string ttlType;
  
  soapMsgIsValid 
    = getTTLInfo(soapMsgFromMsgdrv, ttlHeaderExist, ttl, ttlType);
  cerr << "FIFOSrvDrv::run:ttlHeaderExist = " << ttlHeaderExist << endl;
  cerr << "FIFOSrvDrv::run:ttl = " << ttl << endl;
  
  if (!soapMsgIsValid) {
    std::string invalidReqErrMsg = makeInvalidRequestMessage();
    std::string fileIDOfInvalidReqErrMsg = convertToFileID(invalidReqErrMsg);
    chnlDesc_.write(fileIDOfInvalidReqErrMsg); 
    return;
  }

  if (!ttlHeaderExist) {
    if (_isAsync) {
      ttl = ssmlInfo.getAsyncTTL();
    }
    else {
      ttl = ssmlInfo.getSyncTTL();
    }
  }

  bool targetThreadIsCanceled = false;
  

  if (operationExist) {
    targetThreadIsCanceled = execLocalSyncOperation();
  }
  
  if (targetThreadIsCanceled) {
    // ^CAEg thr_func LZďIꍇB
    soapMsgFromService = makeTimeoutMessage();
    fileIDOfMsgFromService = convertToFileID(soapMsgFromService);
    if (0 > chnlDesc_.write(fileIDOfMsgFromService)) {
      cerr << TAG_ERR << "SrvDrv::run: write failed" << endl;
      return;
    }
  }

  if (_isAsync) {
    // 񓯊̏ꍇ̓X|XXv[ɑB
    asyncResponse();
  }
  else {
    // ̏ꍇ̓bZ[WhCoɃX|XԂB
    if (0 > chnlDesc_.write(fileIDOfMsgFromService)) {
      cerr << TAG_ERR << "SrvDrv::run: write failed" << endl;
      return;
    }
  }
  
}


void 
FIFOSrvDrv::setAsyncFlag(bool isAsync)
{
  _isAsync = isAsync;
}

bool
FIFOSrvDrv::asyncResponse()
{
  // Response Spool Ɍʂ𑗂B
  
  ChannelDescriptor resSpoolPushChnlDesc;
  ChannelSelector* resSpoolPushChnlSelector = new ResSpoolPushChnlSelector();
  if (0 != resSpoolPushChnlSelector->open(resSpoolPushChnlDesc)) {
    cerr << "FIFOSrvDrv::asyncResponse() Chnl open failed" << endl;
    return false;
  }
  
  cout << "send to response spool" << endl;
  if (0 > resSpoolPushChnlDesc.write(fileIDOfMsgFromService)) {
    cerr << "FIFOSrvDrv::asyncResponse: write failed to Service" << endl;
    return false;
  }
  
  std::string fileIDOfMsgFromResSMgr;
  if (0 > resSpoolPushChnlDesc.read(fileIDOfMsgFromResSMgr)) {
    cerr << "FIFOSrvDrv::asyncResponse: read failed from Service" << endl;
    return false;
  }
  
  cerr << "** MsgDrv ** READ From Service=[" 
       << fileIDOfMsgFromResSMgr << "]" << endl;
  
  cout << "send to request queue" << endl;
  
  resSpoolPushChnlSelector->close(resSpoolPushChnlDesc);
  delete resSpoolPushChnlSelector;
  
  
  std::string msgToReqQMgr = "success";
  std::string fileIDOfMsgToReqQMgr = convertToFileID(msgToReqQMgr);
  if (0 > chnlDesc_.write(fileIDOfMsgToReqQMgr)) {
    cerr << TAG_ERR << "FIFOSrvDrv::run: write failed" << endl;
    return false;
  }
  
  return true;
}

