/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Queue.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <algorithm>
#include <unistd.h>
#include <iostream>
#include "ServerCommon.h"
#include "ChannelDescriptor.h"
#include "ForwarderChnlSelector.h"
#include "SrvDrvChnlSelector.h"
#include "Queue.h"

#include "FileIDFunc.h"

using namespace OpenSOAP;

// RXgN^
// Fe[uobNAbvt@C̃pXO
//
Queue::Queue(char filePass[100], bool isFwdQueue)
{
  cerr << "Queue::Queue Constructor" << endl;
  TableRecord tableRecord;
  isFwdQueue_ = isFwdQueue;
  
  queueTable.clear();
  strcpy(tableFile, filePass);

  // R[ht@Cǂݍ݁iȂ΍j
  tin.open(tableFile, ios::in);
  if (tin.fail()) {
    cout << "Create table file" << endl;
  }
  while (!tin.eof() && !tin.fail()) {
    tin >> tableRecord.fileID;
    tableRecord.proccessFlag = INITIAL_STATE;
    if (!tin.eof()) {
      cout << "init: " << tableRecord.fileID << "\t" 
	   << tableRecord.proccessFlag << endl;
      queueTable.push_back(tableRecord);    
    }
  }
  tin.close();

  // MutexCϐCXbh̏
  pthread_mutex_init(&queue_mutex, NULL);
  pthread_cond_init(&queue_cond, NULL);
  for (int thrd = 0; thrd < MAX_POP_THREAD_NUMBER ; thrd++) {
    pthread_create(&pop_thread[thrd], NULL, pop, this);
  }
}


// e[ut@C̏o֐
//
int Queue::fileOut()
{
  tout.open(tableFile, ios::out);
  if (tout.fail()) {
    cout << "Table_file output open error" << endl;
    return 1;
    exit(1);
  }

  for (unsigned int i=0; i<queueTable.size(); i++) {
    tout << queueTable[i].fileID << endl ;
    tout.flush();
    cout << queueTable[i].fileID << "\tFlag: "
	 << queueTable[i].proccessFlag << endl;
  }

  tout.close();
  return 0;
}

// Push\bh
// FiID
//
int Queue::push(string ID)
{
  // [J\̂̃R[hioFfileIDj
  TableRecord tableRecord;

  tableRecord.fileID = ID;
  tableRecord.proccessFlag = int(INITIAL_STATE);

  // mutex̃bN
  int sl = pthread_mutex_lock(&(queue_mutex));
  if (sl == 0) {
    cout << "push thread mutex locked" << endl;
  }

  //  Queue̍Ōɒǉ
  queueTable.push_back(tableRecord);
  
  cout << "push" << endl;
  cout << "vector size = " << queueTable.size() << endl ;
  
  // Queuee[ũt@Co
  int sf = fileOut();
  if (sf != 0) {
    cout << "Table file write error" << endl;
    return 1;
  }

  // PopXbh֏ϐ̑M
  int sc = pthread_cond_signal(&(queue_cond));
  if (sc == 0) {
    cout << "push thread condition value sent" << endl;
  }
  
  // Mutex̉
  int su = pthread_mutex_unlock(&(queue_mutex));
  if (su == 0) {
    cout << "push thread mutex unlocked" << endl;
  }

}

// PopXbh
//
void* Queue::pop(void* arg)
{
  string ID;
  int status;
  pthread_t tid = pthread_self();

  cout << "thread ID: "<< tid << " created" << endl;

  // CX^Xւ̃|C^
  Queue* that = (Queue*)arg;
  if (NULL == that) {
    cerr << "Invalid argument" << endl;
  }

  // PopXbh̃C[v
  while (1) {
    
    // Mutex̃bN
    status = pthread_mutex_lock(&(that->queue_mutex));
    if (status == 0) {
      cout << tid << " pop thread mutex (read) locked" << endl;
    }
    
    // Queue܂ĂȂꍇMutexC
    // Push\bh̏ϐ҂
    //vector<tableRecord>::iterator p = that->queueTable.end();
    int tableSize = that->queueTable.size();
    if (tableSize <= 0 || 
	that->queueTable[tableSize-1].proccessFlag != INITIAL_STATE) {
      cout << tid << " condition value waiting..." << endl;
      status = pthread_cond_wait(&(that->queue_cond), &(that->queue_mutex));
      if (status == 0) {
	cout << tid << " condition value received from puth method" << endl;
      }
    }

    // Queue̐擪R[hitÔȂ́j̓ǂݏo
    vector<TableRecord>::iterator itrRead = that->queueTable.begin();
    while (itrRead->proccessFlag != INITIAL_STATE
	    && itrRead != that->queueTable.end()) itrRead++;
    
    ID = itrRead->fileID;
    cout << "Poped ID: "<< ID << " Flag: " << itrRead->proccessFlag << endl;
    itrRead->proccessFlag = PROGRESS;
    that->fileOut();
    
    // Mutex̉
    status = pthread_mutex_unlock(&(that->queue_mutex));
    if (status == 0) {
      cout << tid << " pop thread mutex (read) unlocked" << endl;
    }
    
    // SrvDrv ւ̓]\bh
    status = that->popOutToNextDrv(ID);
    if (status == 0) {
      cout << tid << " Pop Output Successed" << endl;
    }
    
    // ㏈iQueueRecord̏Ct@Coj
    status = that->queueEraseAndFileOut(ID);
    if (status == 0) {
      cout << tid << " Record Erase Successed" << endl;
    }
  
  }
}

// // SrvDrv ւ̓]\bh
//
bool
Queue::popOutToNextDrv(string fileID) {
  pthread_t tid = pthread_self();
  string nextDrv;
  if (isFwdQueue_) {
    nextDrv = "Fwd";
  }
  else {
    nextDrv = "SrvDrv";
  }
  
  cout << tid << " Async" << nextDrv << " thread wait" << endl;
  
  // Service Driver ̕ԓ҂
  
  ChannelDescriptor nextDrvChnlDesc;
  ChannelSelector* nextDrvChnlSelector;
  if (isFwdQueue_) {
    nextDrvChnlSelector = new ForwarderChnlSelector();
  }
  else {
    nextDrvChnlSelector = new SrvDrvChnlSelector(true);
  }
  if (0 != nextDrvChnlSelector->open(nextDrvChnlDesc)) {
    cerr << "Queue::popOutToNextDrv: " 
	 << nextDrv << "Chnl open failed" << endl;
    return false;
  }
  std::string fileIDOfMsgFromMsgDrv = fileID;
  if (0 > nextDrvChnlDesc.write(fileIDOfMsgFromMsgDrv)) {
    cerr << "Queue::popOutToNextDrv: write failed to " << nextDrv << endl;
    return false;
  }
  cerr << "** Queue ** WRITE To " << nextDrv << "=[" 
       << fileIDOfMsgFromMsgDrv << "]" << endl;
  
  std::string fileIDOfMsgFromNextDrv;
  if (0 > nextDrvChnlDesc.read(fileIDOfMsgFromNextDrv)) {
    cerr << "Queue::popOutToNextDrv: read failed from " << nextDrv << endl;
    return false;
  }
  cerr << "** Queue ** READ From " << nextDrv << "=[" 
       << fileIDOfMsgFromNextDrv << "]" << endl;
  
  nextDrvChnlSelector->close(nextDrvChnlDesc);
  delete nextDrvChnlSelector;
  
  cout << tid << " Async" << nextDrv << " thread end" << endl;
  return true;
}


// ㏈iQueueRecord̏Ct@Coj
//
int Queue::queueEraseAndFileOut(string fileID) {

  int status;
  pthread_t tid = pthread_self();

  // Mutex̍ăbN
  status = pthread_mutex_lock(&(queue_mutex));
  if (status == 0) {
    cout << tid << " pop thread mutex (rewrite) locked" << endl;
  }
  
  // Queuȅς݃R[h̏
  vector<TableRecord>::iterator itrErase = queueTable.begin();
  while (itrErase->fileID != fileID
	  && itrErase != queueTable.end()) itrErase++;
  cout << "File ID: " << itrErase->fileID << " erased" << endl;
  queueTable.erase(itrErase);
  
  cout << "vector size = " << queueTable.size() << endl ;


  // Queuee[ũt@Co
  status = fileOut();
  if (status != 0) {
    cout << "Table file write error" << endl;
  }

  // Mutex̉
  status = pthread_mutex_unlock(&(queue_mutex));
  if (status == 0) {
    cout << tid << " pop thread mutex (rewrite) unlocked" << endl;
  }
}


// fXgN^
//
Queue::~Queue()
{
  cerr << "Queue::~Queue Destructor" << endl;
  
  for (int thrd = 0; thrd < MAX_POP_THREAD_NUMBER ; thrd++) {
    pthread_join(pop_thread[thrd], NULL);
  }
}

void 
Queue::setChnlDesc(const ChannelDescriptor& chnlDesc)
{
  cerr << "Queue::setChnlDesc" << endl;
  chnlDesc_ = chnlDesc;
}

bool 
Queue::execPushOpr()
{
  // MsgDrv 瑗Ăf[^ (t@C ID) ǂݍށB
  string fileIDOfMsgFromMsgDrv;
  if (0 > chnlDesc_.read(fileIDOfMsgFromMsgDrv)) {
    cerr << TAG_ERR << "Queue::execPushOpr: read failed" << endl;
    return false;
  }
  
  cerr << "** Queue ** READ From MsgDrv=[" 
       << fileIDOfMsgFromMsgDrv << "]" << endl;
  
  // L[ɓB
  push(fileIDOfMsgFromMsgDrv);
  
  // L[ɓƂ̊mFbZ[W MsgDrv ɕԂB
  string msgToMsgDrv = "Queue Push success";
  string fileIDOfMsgToMsgDrv = convertToFileID(msgToMsgDrv);
  if (0 > chnlDesc_.write(fileIDOfMsgToMsgDrv)) {
    cerr << TAG_ERR << "Queue::execPushOpr: read failed" << endl;
    return false;
  }
  
  cerr << "** Queue ** WRITE To MsgDrv=[" 
       << fileIDOfMsgToMsgDrv << "]" << endl;
  
  
  return true;
}
    




